@@ -555,7 +555,7 @@ static int HostBitsTestSig01(void)
                     "Host: one.example.org\r\n"
                     "\r\n";
     uint16_t buflen = strlen((char *)buf);
-    Packet *p = SCMalloc(SIZE_OF_PACKET);
+    Packet *p = PacketGetFromAlloc();
     FAIL_IF_NULL(p);
     Signature *s = NULL;
     ThreadVars th_v;
@@ -587,9 +587,8 @@ static int HostBitsTestSig01(void)
 
     DetectEngineThreadCtxDeinit(&th_v, (void *)det_ctx);
     DetectEngineCtxFree(de_ctx);
+    PacketFree(p);
     HostBitsTestShutdown();
-
-    SCFree(p);
     PASS;
 }
 
@@ -736,7 +735,7 @@ static int HostBitsTestSig04(void)
                     "Host: one.example.org\r\n"
                     "\r\n";
     uint16_t buflen = strlen((char *)buf);
-    Packet *p = SCMalloc(SIZE_OF_PACKET);
+    Packet *p = PacketGetFromAlloc();
     if (unlikely(p == NULL))
         return 0;
     Signature *s = NULL;
@@ -773,9 +772,9 @@ static int HostBitsTestSig04(void)
 
     DetectEngineThreadCtxDeinit(&th_v, (void *)det_ctx);
     DetectEngineCtxFree(de_ctx);
+    PacketFree(p);
     HostBitsTestShutdown();
 
-    SCFree(p);
     PASS;
 }
 
@@ -793,9 +792,7 @@ static int HostBitsTestSig05(void)
                     "Host: one.example.org\r\n"
                     "\r\n";
     uint16_t buflen = strlen((char *)buf);
-    Packet *p = SCMalloc(SIZE_OF_PACKET);
-    if (unlikely(p == NULL))
-        return 0;
+    Packet *p = PacketGetFromAlloc();
     Signature *s = NULL;
     ThreadVars th_v;
     DetectEngineThreadCtx *det_ctx = NULL;
@@ -833,10 +830,10 @@ static int HostBitsTestSig05(void)
 
     DetectEngineThreadCtxDeinit(&th_v, (void *)det_ctx);
     DetectEngineCtxFree(de_ctx);
+    PacketFree(p);
 
     HostBitsTestShutdown();
 
-    SCFree(p);
     PASS;
 }
 
@@ -1065,9 +1062,7 @@ static int HostBitsTestSig07(void)
                     "Host: one.example.org\r\n"
                     "\r\n";
     uint16_t buflen = strlen((char *)buf);
-    Packet *p = SCMalloc(SIZE_OF_PACKET);
-    if (unlikely(p == NULL))
-        return 0;
+    Packet *p = PacketGetFromAlloc();
     Signature *s = NULL;
     ThreadVars th_v;
     DetectEngineThreadCtx *det_ctx = NULL;
@@ -1125,9 +1120,8 @@ static int HostBitsTestSig07(void)
     DetectEngineCtxFree(de_ctx);
 
     FLOW_DESTROY(&f);
-
+    PacketFree(p);
     HostBitsTestShutdown();
-    SCFree(p);
     PASS;
 }
 
@@ -1144,9 +1138,7 @@ static int HostBitsTestSig08(void)
                     "Host: one.example.org\r\n"
                     "\r\n";
     uint16_t buflen = strlen((char *)buf);
-    Packet *p = SCMalloc(SIZE_OF_PACKET);
-    if (unlikely(p == NULL))
-        return 0;
+    Packet *p = PacketGetFromAlloc();
     Signature *s = NULL;
     ThreadVars th_v;
     DetectEngineThreadCtx *det_ctx = NULL;
@@ -1221,10 +1213,9 @@ static int HostBitsTestSig08(void)
     DetectEngineCtxFree(de_ctx);
 
     FLOW_DESTROY(&f);
-
+    PacketFree(p);
     HostBitsTestShutdown();
 
-    SCFree(p);
     PASS;
 }
 
@@ -1247,4 +1238,4 @@ void HostBitsRegisterTests(void)
     UtRegisterTest("HostBitsTestSig07", HostBitsTestSig07);
     UtRegisterTest("HostBitsTestSig08", HostBitsTestSig08);
 }
-#endif /* UNITTESTS */
\ No newline at end of file
+#endif /* UNITTESTS */
